﻿using System;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Linq;
using System.Linq.Expressions;

namespace SchedulingTool
{
    public abstract class MyBaseEntity : INotifyPropertyChanged
    {
        #region Fields
        private string _description;
        private string _id;
        private bool _isVisible;
        private string _name;
        #endregion

        #region Properties
        public bool IsVisible
        {
            get { return this._isVisible; }
            set { this.SetProperty(() => this.IsVisible, ref this._isVisible, value); }
        }

        public string Description
        {
            get { return this._description; }
            set { this.SetProperty(() => this.Description, ref this._description, value); }
        }

        public string Id
        {
            get { return this._id; }
            set { this.SetProperty(() => this.Id, ref this._id, value); }
        }

        public string Name
        {
            get { return this._name; }
            set { this.SetProperty(() => this.Name, ref this._name, value); }
        }

        public abstract Object KeyValue { get; }
        #endregion

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;
        #endregion

        protected void NotifyPropertyChanged<TProperty>(Expression<Func<TProperty>> property)
        {
            if (this.PropertyChanged == null)
                return;

            var lambda = (LambdaExpression) property;
            var unaryExpression = lambda.Body as UnaryExpression;
            MemberExpression memberExpression = unaryExpression != null
                                                    ? (MemberExpression) unaryExpression.Operand
                                                    : (MemberExpression) lambda.Body;
            this.PropertyChanged(this, new PropertyChangedEventArgs(memberExpression.Member.Name));
        }

        protected void SetProperty<TProperty>(Expression<Func<TProperty>> property, ref TProperty field, TProperty value)
        {
            field = value;
            this.NotifyPropertyChanged(property);
        }
    }

    public abstract class MyBaseEntityList<T> : ObservableCollection<T> where T : MyBaseEntity
    {
        protected override void MoveItem(int oldIndex, int newIndex)
        {
            throw new InvalidOperationException();
        }

        protected override sealed void SetItem(int index, T item)
        {
            throw new InvalidOperationException();
        }

        protected override void InsertItem(int index, T item)
        {
            if (!this.Any(i => i.KeyValue.Equals(item)))
            {
                base.InsertItem(index, item);
            }
        }

        protected override void OnCollectionChanged(NotifyCollectionChangedEventArgs e)
        {
            base.OnCollectionChanged(e);
            if (e.NewItems != null)
            {
                foreach (MyBaseEntity item in e.NewItems)
                    item.PropertyChanged += this.ItemPropertyChanged;
            }
            if (e.OldItems != null)
            {
                foreach (MyBaseEntity item in e.OldItems)
                    item.PropertyChanged -= this.ItemPropertyChanged;
            }
        }

        private void ItemPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "IsVisible")
            {
                this.OnPropertyChanged(new PropertyChangedEventArgs("IsVisible"));
            }
        }
    }
}