﻿using System;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Windows.Media;

namespace SchedulingTool
{
    public class MyResource : MyBaseEntity
    {
        #region Fields
        private readonly MyAppointmentList _appointments;
        private SolidColorBrush _brush;
        private MyCalendar _calendar;
        private string _daySettingsOverrides;
        private string _daysOfWeek;
        private string _emailAddress;
        private byte? _firstDayOfWeek;
        private string _isLocked;
        private string _primaryCalendarId;
        private string _primaryTimeZoneId;
        #endregion

        #region Properties

        public MyAppointmentList Appointments
        {
            get { return this._appointments; }
        }
        public SolidColorBrush Brush
        {
            get { return this._brush; }
            set { this.SetProperty(() => this.Brush, ref this._brush, value); }
        }

        public ObservableCollection<MyAppointment> Tasks { get; set; }

        public string DaySettingsOverrides
        {
            get { return this._daySettingsOverrides; }
            set { this.SetProperty(() => this.DaySettingsOverrides, ref this._daySettingsOverrides, value); }
        }

        public string DaysOfWeek
        {
            get { return this._daysOfWeek; }
            set { this.SetProperty(() => this.DaysOfWeek, ref this._daysOfWeek, value); }
        }

        public string EmailAddress
        {
            get { return this._emailAddress; }
            set { this.SetProperty(() => this.EmailAddress, ref this._emailAddress, value); }
        }

        public byte? FirstDayOfWeek
        {
            get { return this._firstDayOfWeek; }
            set { this.SetProperty(() => this.FirstDayOfWeek, ref this._firstDayOfWeek, value); }
        }

        public string IsLocked
        {
            get { return this._isLocked; }
            set { this.SetProperty(() => this.IsLocked, ref this._isLocked, value); }
        }

        public string PrimaryCalendarId
        {
            get { return this._primaryCalendarId; }
            set { this.SetProperty(() => this.PrimaryCalendarId, ref this._primaryCalendarId, value); }
        }

        public string PrimaryTimeZoneId
        {
            get { return this._primaryTimeZoneId; }
            set { this.SetProperty(() => this.PrimaryTimeZoneId, ref this._primaryTimeZoneId, value); }
        }

        public MyCalendar Calendar
        {
            get { return this._calendar; }
            set { this.SetProperty(() => this.Calendar, ref this._calendar, value); }
        }
        #endregion

        #region Ctor
        public MyResource()
        {
            this._appointments = new MyAppointmentList();
        }
        #endregion

        #region Methods
        public override object KeyValue
        {
            get { return this.Id; }
        }

        public void AddTask(MyAppointment info)
        {
            info.OwningResourceId = this.Id;
            info.OwningCalendarId = this.Calendar.Id;
            info.OwningCalendar = this.Calendar;
            info.OwningResource = this;
            info.IsVisible = true;
            info.IsTimeZoneNeutral = true;
        }
        #endregion
      
    }

    public sealed class MyResourceList : MyBaseEntityList<MyResource>
    {
        #region Fields
        private const int RandomSeed = 7;
        private const string Calendar = "Calendar";
        private readonly Random _random;
        private readonly ObservableCollection<MyCalendar> _resourceCalendars;
        #endregion

        #region Properties
        public ObservableCollection<MyCalendar> ResourceCalendars
        {
            get { return this._resourceCalendars; }
        }
        #endregion

        #region Ctor
        public MyResourceList()
        {
            this._random = new Random(RandomSeed);
            this._resourceCalendars = new ObservableCollection<MyCalendar>();
        }
        #endregion

        #region Overrides
        protected override void InsertItem(int index, MyResource item)
        {
            var brush = new SolidColorBrush(this.GenerateColor());
            brush.Freeze();
            item.Brush = brush;
            item.IsVisible = true;
            item.Calendar = new MyCalendar
                                {
                                    Id = item.Id + Calendar,
                                    OwningResourceId = item.Id,
                                    BaseColor = brush.Color,
                                    IsVisible = true
                                };
            base.InsertItem(index, item);
        }

        protected override void OnCollectionChanged(NotifyCollectionChangedEventArgs e)
        {
            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    foreach (MyResource newItem in e.NewItems)
                    {
                        this.ResourceCalendars.Add(newItem.Calendar);
                    }
                    break;
                case NotifyCollectionChangedAction.Remove:
                    foreach (MyResource oldItem in e.OldItems)
                    {
                        this.ResourceCalendars.Remove(oldItem.Calendar);
                    }
                    break;
                case NotifyCollectionChangedAction.Reset:
                    this.ResourceCalendars.Clear();
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }

            base.OnCollectionChanged(e);
        }
        #endregion

        #region Private Methods
        private Color GenerateColor()
        {
            var colorBytes = new byte[3];
            colorBytes[0] = (byte) (this._random.Next(128) + 127);
            colorBytes[1] = (byte) (this._random.Next(128) + 127);
            colorBytes[2] = (byte) (this._random.Next(128) + 127);
            var color = new Color {A = 255, R = colorBytes[0], B = colorBytes[1], G = colorBytes[2]};
            return color;
        }
        #endregion
    }
}